<?php
namespace Models;

// модель для чеков
class Receipts extends \DB\SQL\Mapper {
	public $db, $f3, $fr;

	public function __construct($db, $f3) {
		parent::__construct($db, 'receipts' ); // подключаемся к таблице чеков
		$this->db=$db;
		$this->f3=$f3;
		$this->fr=new \Models\Fr($db, $f3);
	}

	// Информация об открываемой странице
	function page_data($page='sales') {
		$pages=array(
			'sales'=>array(
				'title'=>'Продажа',
				'inc'=>'receipts_sales.htm'
			),
			'return'=>array(
				'title'=>'Возврат',
				'inc'=>'receipts_return.htm'
			),
			'search'=>array(
				'title'=>'Поиск чеков',
				'inc'=>'receipts_search.htm'
			),
			'paymentmethods'=>array(
				'title'=>'Оплата чека',
				'inc'=>'receipts_paymentmethods.htm'
			),
			'show'=>array(
				'title'=>'Просмотр чека',
				'inc'=>'receipts_show.htm'
			),
			'change_price'=>array(
				'title'=>'Редактирование цены продажи',
				'inc'=>'receipts_change_price.htm'
			),
			'cashin'=>array(
				'title'=>'Внесение денег в кассу',
				'inc'=>'receipts_cashin.htm'
			),
			'cashout'=>array(
				'title'=>'Изъятие денег из кассы',
				'inc'=>'receipts_cashout.htm'
			),
			'reports'=>array(
				'title'=>'Смены и отчеты',
				'inc'=>'receipts_reports.htm'
			),
			'new'=>array(
				'title'=>'Открытие нового чека',
				'inc'=>'receipts_new.htm'
			),
			'switch'=>array(
				'title'=>'Переключение на другой чек',
				'inc'=>'receipts_switch.htm'
			),
			'all_open'=>array(
				'title'=>'Открытые чеки',
				'inc'=>'receipts_all_open.htm'
			)
		);
		$pages[$page]['page']=$page;
		return $pages[$page];
	}

	// Выбираем чеки
	function select_receipts($query=null, $options=null) {
		if(!$query) {
			$query=array('id>?', 0); // чтобы хоть что-то вставить в фильтры
		}

		// формируем сортировку и лимиты
		$option='';
		if($options['order']) {
			$option.=' ORDER BY '.$options['order'];
		}
		if($options['limit']) {
			$option.=' LIMIT '.$options['limit'];
		}
		// собственно сам запрос
		$receipts=array();
		$sql="SELECT r.*, u.name, u.short_name, g.title FROM receipts AS r LEFT JOIN users AS u ON u.id=r.user_id LEFT JOIN usergroups AS g ON g.id=u.group_id WHERE (".$query[0].") AND r.time>0 ".$option;
		$receipts=$this->db->exec($sql, $query[1]);
		$receiptlines=new \Models\Receiptlines($this->db, $this->f3);
		foreach ($receipts as $n => $r) {
			$receipts[$n]['receiptlines']=$receiptlines->select_lines(array('receipt_id=?', $r['id']));
			if($r['type']==$this->f3->get('receipt_sales') || $r['type']==$this->f3->get('receipt_return')) {
				if($receipts[$n]['lines_count']==null) {
					$receipts[$n]['lines_count']=$receipts[$n]['lines_count_add']=count($receipts[$n]['receiptlines']);
				}
			}
			else {
				unset($receipts[$n]['lines_count']);
			}
			$receipts[$n]['paymentmethods']=json_decode($r['payment'], true);
		}
		return $receipts;
	}

	// Узнаем окрытый чек
	function get_open_receipt($rm, $store, $open=true) {
		$this->reset();
		$user_id=$this->f3->get('SESSION.user.id');
		$receipt_id=$this->f3->get('SESSION.receipt_id');
		$this->load(array('rm=? AND store=? AND time=0 AND user_id=?', array($rm, $store, $user_id)));
		if($this->dry()) {
			if(!$open) {
				return array(
					'success'=>false,
					'txt'=>'not_found'
				);
			}
			$receipt=array(
				'rm'=>$rm,
				'store'=>$store,
				'user_id'=>$user_id,
				'time'=>0
			);
			$this->copyFrom($receipt);
			if($this->insert()) {
				$receipt['id']=$this->get('_id');
				return array(
					'success'=>true,
					'txt'=>'save_successful',
					'receipt'=>$receipt,
					'open'=>array()
				);
			}
			else {
				return array(
					'success'=>false,
					'txt'=>'db_error'
				);
			}
		}
		else {
			$receipt=array();
			$open=array();
			do {
				$ticket=$this->cast();
				if($ticket['id']==$receipt_id) {
					$receipt=$ticket;
				}
				else {
					$open[]=$ticket;
				}
			}
			while($this->skip());
			if(empty($receipt)) {
				$receipt=$open[0];
			}
			if($receipt['partner_id']>0) {
				$p=new \Models\Partners($this->db, $this->f3);
				$partner=$p->select_partner(array('partner_id=?', $receipt['partner_id']), array('limit'=>1));
				$receipt['partner']=$partner[0];
			}
			return array(
				'success'=>true,
				'receipt'=>$receipt,
				'open'=>$open
			);
		}
	}

	// Ищем чеки
	function search($id, $changes, $barcode, $dates, $type, $start=0, $limit=30) {
		$where='';
		$val=array();
		if($id) {
			$where='r.id=?';
			$val[]=$id;
		}
		if($changes) {
			$where='r.changes=?';
			$val[]=$changes;
		}
		if($dates) {
			if($where!='')$where.=' AND ';
			$where.='r.time>=? AND r.time<=?';
			$val[]=$dates[0];
			$val[]=$dates[1]+3600*24;
		}
		if($type) {
			if($where!='')$where.=' AND ';
			$where.='r.type=?';
			$val[]=$type;
		}
		if($barcode) {
			if($where!='') $where.=' AND ';
			$p=new \Models\Products($this->db);
			$products=$p->search($barcode, null, null, true);
			if($products['success']) {
				$product_id=$products['products'][0]['id'];
				$receiptlines=new \Models\Receiptlines($this->db, $this->f3);
				$lines=$receiptlines->select_lines(array('product_id=?', $product_id));
				if($lines) {
					$bwhere='';
					foreach ($lines as $l) {
						if($bwhere!='')$bwhere.=' OR ';
						$bwhere.=' r.id=? ';
						$val[]=$l['receipt_id'];
					}
					if($bwhere!='') {
						$where.=' ('.$bwhere.') ';
					}
				}
				else {
					$where.='r.id=?';
					$val[]=-1;
				}
			}
			else {
				$where.='r.id=?';
				$val[]=-1;
			}
		}
		if(!$where) {
			$where='r.id>?';
			$val[]=0;
		}
		$receipts=$this->select_receipts(array($where, $val), array('order'=>'time DESC', 'limit'=>$start.', '.$limit));
		$sql="SELECT COUNT(*) AS count FROM receipts AS r WHERE ".$where;
		$res=$this->db->exec($sql, $val);
		$count=$res[0]['count'];

		return array(
			'success'=>true,
			'count'=>$count,
			'receipts'=>$receipts
		);
	}

	// Проверка правильности заполнения контактных данных покупателя
	function client_validate($client) {
		$result=false;
		if(mb_strpos($client, '@')===false) {
			// Проверяем телефон
			if(preg_match("/\+79[0-9]{9}/", $client))$result=true;
		}
		else {
			// проверяем e-mail
			if(filter_var($client, FILTER_VALIDATE_EMAIL))$result=true;
		}
		return $result;
	}

	// Пробиваем чек
	function add_ticket($receipt_id, $change, $type, $paymentmethod, $client_sum, $sum, $lines, $client, $discount=0, $partner_id=0, $sales_receipts=0) {
		if($client && !$this->client_validate($client)) {
			return array(
				'success'=>false,
				'txt'=>'incorrect_email_or_phone'
			);
		}
		$lines_count=count($lines);
		if($lines_count==0) {
			return array(
				'success'=>false,
				'txt'=>'incorrect_email_or_phone'
			);
		}

		// Формируем данные оплаты
		$payment=array();
		foreach ($paymentmethod as $p_id) {
			$payment[]=array('method'=>$p_id, 'sum'=>$client_sum[$p_id]);
		}

		// формируем данные чека
		$receipt=array(
			'id'=>$receipt_id,
			'user_id'=>$this->f3->get('SESSION.user.id'),
			'changes'=>$change['id'],
			'changes_data'=>json_encode($change),
			'time'=>microtime(true),
			'type'=>$type,
			'payment'=>json_encode($payment),
			'sum'=>$sum,
			'client'=>$client,
			'partner_id'=>$partner_id,
			'lines_count'=>$lines_count
		);
		
		$this->reset();
		$this->load(array('id=?', $receipt_id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->copyFrom($receipt);
		$this->fr->ticket($receipt, $lines, $discount, $sales_receipts);
		if(!$this->update()) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}


		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}

	function ticket_return($rm, $store, $change, $post) {
		$lines_count=count($post['receiptlines']);
		if($lines_count==0) {
			return array(
				'success'=>false,
				'txt'=>'incorrect_email_or_phone'
			);
		}

		// Формируем данные оплаты
		$payment=array();
		foreach ($post['paymentmethod'] as $p_id) {
			$payment[]=array('method'=>$p_id, 'sum'=>$post['client_sum'][$p_id]);
		}

		// формируем данные чека
		$receipt=array(
			'user_id'=>$this->f3->get('SESSION.user.id'),
			'rm'=>$rm,
			'store'=>$store,
			'changes'=>$change['id'],
			'changes_data'=>json_encode($change),
			'time'=>0,
			'type'=>$post['type'],
			'payment'=>json_encode($payment),
			'sum'=>$post['sum'],
			'client'=>'',
			'partner_id'=>$post['partner_id'],
			'lines_count'=>$lines_count
		);
		$this->reset();
		$this->copyFrom($receipt);
		if(!$this->insert()) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		$receipt['id']=$this->get('_id');

		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'id'=>$receipt['id'],
			'receipt'=>$receipt
		);
	}

	// Проводим чек возврата
	function ticket_ready($receipt, $post) {
		$this->reset();
		$this->load(array('id=?', $receipt['id']), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->time=microtime(true);
		$this->fr->ticket($receipt, $post['receiptlines'], $post['discount']);
		if($this->update()) {
			return array(
				'success'=>true,
				'txt'=>'save_successful'
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Чек смены
	function ticket_change($api_data, $change, $type, $paymentmethod, $sum=0) {
		$this->reset();
		// Формируем данные оплаты
		$payment[]=array('method'=>$paymentmethod, 'sum'=>$sum);
		// формируем данные чека
		$receipt=array(
			'store'=>$api_data['store_id'],
			'rm'=>$api_data['rm_id'],
			'user_id'=>$this->f3->get('SESSION.user.id'),
			'changes'=>$change['id'],
			'changes_data'=>json_encode($change),
			'time'=>microtime(true),
			'type'=>$type,
			'payment'=>json_encode($payment),
			'sum'=>$sum,
			'client'=>'',
			'partner_id'=>0
		);
		$this->copyFrom($receipt);
		if($this->insert()) { // сохраняем
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'id'=>$this->get('_id')
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Удаление чека
	function delete_receipt($id) {
		$this->reset();
		$this->load(array('id=?', $id), array('limit'=>1));
		$this->erase();
		$this->db->exec("DELETE FROM receiptlines WHERE receipt_id=?", $id);
	}

	// Внесение и изъятие денег из кассы
	function cashin($api_data, $change, $pay, $sum, $type, $create_receipt=0, $description='') {
		$this->fr->cashin($change['id'], $sum);
		$receipt=array(
			'store'=>$api_data['store_id'],
			'rm'=>$api_data['rm_id'],
			'user_id'=>$this->f3->get('SESSION.user.id'),
			'changes'=>$change['id'],
			'changes_data'=>json_encode($change),
			'time'=>microtime(true),
			'type'=>$type,
			'payment'=>json_encode(array(array('method'=>$pay, 'sum'=>$sum))),
			'sum'=>$sum,
			'client'=>'',
			'create_receipt'=>$create_receipt,
			'description'=>$description
		);
		$this->reset();
		$this->copyFrom($receipt);
		$this->save();
		if($this->get('_id')) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'form_reset'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Статистика по смене
	function change_statistic($change) {
		$receipts=$this->select_receipts(array('changes=? AND type!=? AND type!=?', array($change, $this->f3->get('receipt_change_open'), $this->f3->get('receipt_change_close'))));
		$report=array(
			$this->f3->get('receipt_sales')=>array('sum'=>0, 'count'=>0),
			$this->f3->get('receipt_return')=>array('sum'=>0, 'count'=>0),
			$this->f3->get('receipt_cashin')=>array('sum'=>0, 'count'=>0),
			$this->f3->get('receipt_cashout')=>array('sum'=>0, 'count'=>0)
		);
		foreach ($receipts as $r) {
			$report[$r['type']]['count']+=1;
			$report[$r['type']]['sum']+=$r['sum'];
			$pays=json_decode($r['payment'], true);
			foreach ($pays as $p) {
				if($p['sum']>$r['sum']) {
					$p['sum']=$r['sum'];
				}
				$report[$r['type']]['pay'][$p['method']]+=$p['sum'];
			}
		}
		return $report;
	}

	// Отчет по секциям
	function rep_sections() {
		$fr_type=$this->fr->get_fr()['type'];
		if($fr_type==1) {
			$this->fr->rep_sections();
			return array(
				'success'=>true,
				'txt'=>'report_is_ready'
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'fr_only'
			);
		}
	}

	// Печать копии последнего чека
	function print_last() {
		if($this->fr->sets['active']) {
			$this->fr->c();
			return array(
				'success'=>true,
				'txt'=>'request_has_been_sent'
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'kkm_is_not_activity'
			);
		}
	}

	// Открытие нового пустого чека
	function open_new($rm, $store) {
		$user_id=$this->f3->get('SESSION.user.id');
		$this->reset();
		$receipt=array(
			'rm'=>$rm,
			'store'=>$store,
			'user_id'=>$user_id,
			'time'=>0
		);
		$this->copyFrom($receipt);
		if($this->insert()) {
			$this->f3->set('SESSION.receipt_id', $this->get('_id'));
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.receiptlines',
				'reload_menu'=>true,
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Переключение между на другой чек
	function switch_receipt($id) {
		if(!$id) {
			return array(
				'success'=>false,
				'txt'=>'not_enough_data'
			);
		}
		$this->reset();
		$this->load(array('id=?', $id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->f3->set('SESSION.receipt_id', $this->id);
		return array(
			'success'=>true,
			'txt'=>'save_successful',
			'reload'=>'.receiptlines',
			'reload_menu'=>true,
			'close_all_window'=>true
		);
	}

	// Добавление клиента к чеку
	function add_partner($receipt_id, $partner) {
		$this->reset();
		$this->load(array('id=? AND time=0', $receipt_id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->partner_id=$partner['partner_id'];
		if($partner['email']) {
			$contact=$partner['email'];
		}
		else {
			$contact=$partner['phone'];
		}
		$this->client=$contact;
		$this->discount=$partner['discount'];
		if($this->update()!==false) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.receiptlines',
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Убрать клиента из чека
	function remove_partner($id) {
		$this->reset();
		$this->load(array('id=?', $id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->partner_id='';
		$this->discount=0;
		if($this->update()) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.receiptlines',
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Сохранение произвольной скидки в чек
	function set_discount($discount) {
		if(is_numeric($discount) && $discount>=0 && $discount<=100) {
			$this->reset();
			$id=$this->f3->get('SESSION.receipt_id');
			$this->load(array('id=?', $id), array('limit'=>1));
			if($this->dry()) {
				return array(
					'success'=>false,
					'txt'=>'not_found'
				);
			}
			$this->discount=$discount;
			if($this->update()!==false) {
				return array(
					'success'=>true,
					'txt'=>'save_successful'
				);
			}
			else {
				return array(
					'success'=>false,
					'txt'=>'db_error'
				);
			}
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'incorrect_discount'
			);
		}
	}

	// Поиск всех открытых чеков
	function all_open($start=0, $limit=30) {
		$sql="SELECT r.*, u.name, u.short_name, g.title FROM receipts AS r LEFT JOIN users AS u ON u.id=r.user_id LEFT JOIN usergroups AS g ON g.id=u.group_id WHERE r.time=0 LIMIT ".$start.', '.$limit;
		$receipts=$this->db->exec($sql);
		$sql="SELECT COUNT(*) AS count FROM receipts AS r WHERE time=0";
		$res=$this->db->exec($sql);
		$count=$res[0]['count'];
		return array(
			'success'=>true,
			'count'=>$count,
			'receipts'=>$receipts
		);
	}

	// Перехват чека
	function intercept($rm, $store, $id) {
		$this->reset();
		$this->load(array('id=? AND time=0', $id), array('limit'=>1));
		if($this->dry()) {
			return array(
				'success'=>false,
				'txt'=>'not_found'
			);
		}
		$this->rm=$rm;
		$this->store=$store;
		$this->user_id=$this->f3->get('SESSION.user.id');
		if($this->update()!==false) {
			return array(
				'success'=>true,
				'txt'=>'save_successful',
				'reload'=>'.receiptlines',
				'reload_menu'=>true,
				'close_all_window'=>true
			);
		}
		else {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
	}

	// Получаем открытые не пустые чеки
	function get_not_empty($rm, $store) {
		$sql="SELECT COUNT(r.id) AS count FROM `receipts` AS `r` JOIN `receiptlines` AS `l` ON l.receipt_id=r.id WHERE r.time=0 AND r.rm=? AND r.store=? GROUP BY (l.receipt_id)";
		$val=array($rm, $store);
		$res=$this->db->exec($sql, $val);
		return $res[0]['count'];
	}

	// Удаляем открытые чеки
	function delete_open($rm, $store) {
		// Список открытых чеков
		$receipts=$this->db->exec("SELECT `id` FROM `receipts` WHERE `time`=0 AND `rm`=? AND `store`=?", array($rm, $store));
		if(!empty($receipts)) {
			$in='';
			$val=array();
			foreach ($receipts as $r) {
				if(!empty($val)) {
					$in.=',';
				}
				$in.='?';
				$val[]=$r['id'];
			}
			$sql="DELETE FROM `receipts` WHERE `id` IN (".$in.")";
			$res=$this->db->exec($sql, $val);
			if($res===false) {
				return array(
					'success'=>false,
					'txt'=>'db_error'
				);
			}
			$sql="DELETE FROM `receiptlines` WHERE `receipt_id` IN (".$in.")";
			$res=$this->db->exec($sql, $val);
			if($res===false) {
				return array(
					'success'=>false,
					'txt'=>'db_error'
				);
			}
		}

		return array(
			'success'=>true
		);
	}
}
